"""utils package

Shared helper re-exports for the number-guessing demo.
"""

from utils.helpers import parse_int_in_range, try_parse_json


__all__ = [
    'parse_int_in_range',
    'try_parse_json',
]
